using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace SuperScooter
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Level level;
        Enemy1[] enemies1;
        Enemy2[] enemies2;
        player player1;

        public const int SHIFT = 6;
        public const int CLIENT_WIDTH = 800;
        public const int CLIENT_HEIGHT = 600;
        public const int MAX_LEVEL = 4;
        public const int DEFAULT_TIME_LIMIT = 100;
        public const int START_LIVES = 4;
        public const int SPRITE_CHOICE_COUNT = 7;
        public const int INITIAL_LEVEL = 1;

        #region Menu
        MMenu mainMenu;
        string _playGame = "Play Game";
        string _instructions = "Instructions";
        string _options = "Options";
        string _credits = "Credits";
        string _quit = "Quit";

        string _blank = "";
        string _next = "Next";
        string _prev = "Prev";
        string _returnToMain = "Return to Main Menu";
        MMenu instructionsMenu;
            int instructionPanelCount = 4;
            int currentInstructionPanel = 1;
            Texture2D[] instructionPanels;

        string _selectSpriteColor = "Select Sprite Color";
        string _soundOptions = "Sound Effects";
        string _musicOptions = "Background Music";
        MMenu optionsMenu;
            string _on = "On";
            string _off = "Off";
            MMenu soundOptionsMenu;
            MMenu musicOptionsMenu;
            MMenu colorOptionsMenu;
            Texture2D[] spriteChoices = new Texture2D[SPRITE_CHOICE_COUNT];
            string spriteColor = "";
            string _brown = "Brown";
            string _red = "Red";
            string _orange = "Orange";
            string _yellow = "Yellow";
            string _green = "Green";
            string _blue = "Blue";
            string _purple = "Purple";
            string _reset = "Reset";
            string _save = "Save";

        MMenu creditsMenu;
            int creditsPanelCount = 2;
            int currentCreditsPanel = 1;
            Texture2D[] creditsPanels;

        bool inMainMenu = true;
        bool inInstructions = false;
        bool inCredits = false;
        bool inOptions = false;
        bool inColorOptions = false;
        bool inSoundOptions = false;
        bool inMusicOptions = false;
        #endregion

        #region Flags
        bool pause = false;
        bool won = false;
        bool gameOverWin = false;
        bool gameOverLose = false;
        bool levelActive = false;
        public bool clingOn = false;
        public bool enemiesOn = true;
        private bool playerDead = false;
        private bool playSoundEffects = true;
        #endregion

        #region Level Control
       
        int currentLevel = INITIAL_LEVEL;
        int totalScore = 0;
        int levelScore = 0;
        float timer = DEFAULT_TIME_LIMIT;
        float[] timers = new float[MAX_LEVEL];
        int lives = START_LIVES;
        #endregion

        #region Sounds Variables
        //Sound music;
        Song[] bgMusic = new Song[MAX_LEVEL + 1];
        Hashtable soundEffectNames = new Hashtable();
        SoundEffect[] soundEffects;
        #endregion

        SpriteFont[] Fonts = new SpriteFont[20];

        KeyboardState previousKeyboardState;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();

            graphics.PreferredBackBufferWidth = CLIENT_WIDTH;
            graphics.PreferredBackBufferHeight = CLIENT_HEIGHT;
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            //Load fonts
            Fonts[0] = Content.Load<SpriteFont>(@"Fonts/defaultFont");
            Fonts[1] = Content.Load<SpriteFont>(@"Fonts/happy18");
            Fonts[2] = Content.Load<SpriteFont>(@"Fonts/happy30");
            Fonts[3] = Content.Load<SpriteFont>(@"Fonts/happy36");
            Fonts[4] = Content.Load<SpriteFont>(@"Fonts/happy48");
            Fonts[5] = Content.Load<SpriteFont>(@"Fonts/happy60");
            Fonts[6] = Content.Load<SpriteFont>(@"Fonts/happy72");
            Fonts[7] = Content.Load<SpriteFont>(@"Fonts/cambria18");

            timers[0] = 31;
            timers[1] = 46;
            timers[2] = 61;
            timers[3] = 61;

            //Set up sounds
            bgMusic[0] = Content.Load<Song>(@"Sound/BackMusic2");

            for (int l = 1; l <= MAX_LEVEL; l++)
            {
                string name = "Sound/level" + l;
                bgMusic[l] = Content.Load<Song>(@name);
            }

            MediaPlayer.Play(bgMusic[0]);
            MediaPlayer.IsRepeating = true;

            soundEffects = new SoundEffect[4];
            soundEffectNames["jump"] = 0;
            soundEffects[(int)soundEffectNames["jump"]] = Content.Load<SoundEffect>(@"Sound/Jump");
            soundEffectNames["kill"] = 1;
            soundEffects[(int)soundEffectNames["kill"]] = Content.Load<SoundEffect>(@"Sound/Kill");
            soundEffectNames["die"] = 2;
            soundEffects[(int)soundEffectNames["die"]] = Content.Load<SoundEffect>(@"Sound/Die");
            soundEffectNames["redo"] = 3;
            soundEffects[(int)soundEffectNames["redo"]] = Content.Load<SoundEffect>(@"Sound/Scratch");



            Rectangle windowBounds = Window.ClientBounds;



            //Setup the menus
            Texture2D menuBG = Content.Load<Texture2D>(@"Textures/mainMenuBG");
            Texture2D acorn = Content.Load<Texture2D>(@"Textures/acorn");
            Vector2 menuAnchor = new Vector2(100, 250);
            Color normalColor = new Color(117, 87, 58);//brown color
            Color selectedColor = Color.White;

            mainMenu = new MMenu(menuAnchor, menuBG, acorn, Fonts[3], Fonts[4]);
            mainMenu.MenuItemPaddingCoefficient = 0.1f;
            mainMenu.AbsolutePadding = false;
            mainMenu.NormalColor = normalColor;
            mainMenu.SelectedColor = selectedColor;
            mainMenu.AddMenuItem(_playGame);
            mainMenu.AddMenuItem(_instructions);
            mainMenu.AddMenuItem(_options);
            mainMenu.AddMenuItem(_credits);
            mainMenu.AddMenuItem(_quit);
            mainMenu.Visible = true;
            inMainMenu = true;

            instructionsMenu = new MMenu(menuAnchor, menuBG, acorn, Fonts[2], Fonts[3]);
            instructionsMenu.Visible = false;
            inInstructions = false;
            instructionsMenu.NormalColor = normalColor;
            instructionsMenu.SelectedColor = selectedColor;
            instructionsMenu.AddMenuItem(_prev, new Vector2(70, 500), Color.White);
            instructionsMenu.AddMenuItem(_next, new Vector2(325, 500), Color.White);
            instructionsMenu.AddMenuItem(_returnToMain, new Vector2(60, 545), Color.White);

            instructionPanels = new Texture2D[instructionPanelCount];
            instructionPanels[0] = Content.Load<Texture2D>(@"Textures/Menus/instruction1");
            instructionPanels[1] = Content.Load<Texture2D>(@"Textures/Menus/instruction2");
            instructionPanels[2] = Content.Load<Texture2D>(@"Textures/Menus/instruction3");
            instructionPanels[3] = Content.Load<Texture2D>(@"Textures/Menus/instruction4");

            optionsMenu = new MMenu(new Vector2(menuAnchor.X + 10, menuAnchor.Y), menuBG, acorn, Fonts[2], Fonts[3]);
            optionsMenu.Visible = false;
            inOptions = false;
            optionsMenu.MenuItemPaddingCoefficient = 0.1f;
            optionsMenu.AbsolutePadding = false;
            optionsMenu.NormalColor = normalColor;
            optionsMenu.SelectedColor = selectedColor;
            optionsMenu.AddMenuItem(_selectSpriteColor);
            optionsMenu.AddMenuItem(_soundOptions);
            optionsMenu.AddMenuItem(_musicOptions);
            optionsMenu.AddMenuItem(_returnToMain, new Vector2(60, 500), Color.White);

            colorOptionsMenu = new MMenu(new Vector2(menuAnchor.X + 60, menuAnchor.Y), menuBG, acorn, Fonts[1], Fonts[2]);
            colorOptionsMenu.Visible = false;
            inColorOptions = false;
            colorOptionsMenu.MenuItemPaddingCoefficient = 0.1f;
            colorOptionsMenu.AbsolutePadding = false;
            colorOptionsMenu.NormalColor = normalColor;
            colorOptionsMenu.SelectedColor = selectedColor;
            colorOptionsMenu.AddMenuItem(_brown);
            colorOptionsMenu.AddMenuItem(_red);
            colorOptionsMenu.AddMenuItem(_orange);
            colorOptionsMenu.AddMenuItem(_yellow);
            colorOptionsMenu.AddMenuItem(_green);
            colorOptionsMenu.AddMenuItem(_blue);
            colorOptionsMenu.AddMenuItem(_purple);
            colorOptionsMenu.AddMenuItem(_reset, new Vector2(70, 550), Color.White);
            colorOptionsMenu.AddMenuItem(_save, new Vector2(325, 550), Color.White);

            //load sprite color option choices
            spriteChoices[0] = Content.Load<Texture2D>("Textures/squirrel");
            spriteChoices[1] = Content.Load<Texture2D>("Textures/squirrel_red");
            spriteChoices[2] = Content.Load<Texture2D>("Textures/squirrel_orange");
            spriteChoices[3] = Content.Load<Texture2D>("Textures/squirrel_yellow");
            spriteChoices[4] = Content.Load<Texture2D>("Textures/squirrel_green");
            spriteChoices[5] = Content.Load<Texture2D>("Textures/squirrel_blue");
            spriteChoices[6] = Content.Load<Texture2D>("Textures/squirrel_purple");

            soundOptionsMenu = new MMenu(new Vector2(menuAnchor.X + 330, menuAnchor.Y + 50), null, acorn, Fonts[1], Fonts[2]);
            soundOptionsMenu.Visible = false;
            inSoundOptions = false;
            soundOptionsMenu.MenuItemPaddingCoefficient = 0.1f;
            soundOptionsMenu.AbsolutePadding = false;
            soundOptionsMenu.NormalColor = normalColor;
            soundOptionsMenu.SelectedColor = selectedColor;
            soundOptionsMenu.AddMenuItem(_on);
            soundOptionsMenu.AddMenuItem(_off);

            musicOptionsMenu = new MMenu(new Vector2(menuAnchor.X + 390, menuAnchor.Y + 100), null, acorn, Fonts[1], Fonts[2]);
            musicOptionsMenu.Visible = false;
            inMusicOptions = false;
            musicOptionsMenu.MenuItemPaddingCoefficient = 0.1f;
            musicOptionsMenu.AbsolutePadding = false;
            musicOptionsMenu.NormalColor = normalColor;
            musicOptionsMenu.SelectedColor = selectedColor;
            musicOptionsMenu.AddMenuItem(_on);
            musicOptionsMenu.AddMenuItem(_off);

            creditsMenu = new MMenu(menuAnchor, menuBG, acorn, Fonts[2], Fonts[3]);
            creditsMenu.Visible = false;
            inCredits = false;
            creditsMenu.NormalColor = normalColor;
            creditsMenu.SelectedColor = selectedColor;
            creditsMenu.AddMenuItem(_prev, new Vector2(70, 500), Color.White);
            creditsMenu.AddMenuItem(_next, new Vector2(325, 500), Color.White);
            creditsMenu.AddMenuItem(_returnToMain, new Vector2(60, 545), Color.White);

            creditsPanels = new Texture2D[creditsPanelCount];
            creditsPanels[0] = Content.Load<Texture2D>("Textures/Menus/credits1");
            creditsPanels[1] = Content.Load<Texture2D>("Textures/Menus/credits2");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            #region:input
            KeyboardState currentKeyboardState = Keyboard.GetState();

            if (currentKeyboardState.IsKeyDown(Keys.LeftControl) && currentKeyboardState.IsKeyDown(Keys.LeftShift) && currentKeyboardState.IsKeyDown(Keys.E))
            {
                enemiesOn = false;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.LeftControl) && currentKeyboardState.IsKeyDown(Keys.LeftAlt) && currentKeyboardState.IsKeyDown(Keys.E))
            {
                enemiesOn = true;
            }

            if (currentKeyboardState.IsKeyDown(Keys.LeftControl) && currentKeyboardState.IsKeyDown(Keys.LeftShift) && currentKeyboardState.IsKeyDown(Keys.C))
            {
                clingOn = true;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.LeftControl) && currentKeyboardState.IsKeyDown(Keys.LeftAlt) && currentKeyboardState.IsKeyDown(Keys.C))
            {
                clingOn = false;
            }

            if (currentKeyboardState.IsKeyDown(Keys.M) && !previousKeyboardState.IsKeyDown(Keys.M))
            {
                MediaPlayer.Volume = (MediaPlayer.Volume == 0) ? 100 : 0;

                playSoundEffects = !playSoundEffects;
            }

            if (currentKeyboardState.IsKeyDown(Keys.P) && !previousKeyboardState.IsKeyDown(Keys.P))
                pause = !pause;

            if (inMainMenu)
            {
                mainMenu.Visible = true;
                playerDead = false;
                currentInstructionPanel = 1;
                currentCreditsPanel = 1;
                currentLevel = INITIAL_LEVEL;

                if (currentKeyboardState.IsKeyDown(Keys.Down) && !previousKeyboardState.IsKeyDown(Keys.Down))
                {
                    mainMenu.SelectNext();
                }
                else if (currentKeyboardState.IsKeyDown(Keys.Up) && !previousKeyboardState.IsKeyDown(Keys.Up))
                {
                    mainMenu.SelectPrev();
                }

                if (currentKeyboardState.IsKeyDown(Keys.Enter) && !previousKeyboardState.IsKeyDown(Keys.Enter))
                {
                    mainMenu.Visible = false;

                    if (mainMenu.CurrentlySelectedItem == _playGame)
                    {
                        inMainMenu = false;
                        inCredits = false;
                        gameOverWin = false;
                        gameOverLose = false;
                        won = false;
                        pause = false;

                        totalScore = 0;
                        levelScore = 0;
                        timer = timers[currentLevel - 1];
                        lives = START_LIVES;

                        MediaPlayer.Play(bgMusic[currentLevel]);
                        level = new Level(this, currentLevel);
                        level.setCheat(clingOn);
                        level.setEnemies(enemiesOn);
                        level.LoadContent(this, out enemies1, out enemies2, ref player1, "squirrel" + ((spriteColor != "") ? "_" + spriteColor : ""));
                        levelActive = true;
                    }

                    else if (mainMenu.CurrentlySelectedItem == _instructions)
                    {
                        inInstructions = true;
                        inMainMenu = false;
                        inCredits = false;
                        inOptions = false;
                    }
                    else if (mainMenu.CurrentlySelectedItem == _options)
                    {
                        inOptions = true;
                        inMainMenu = false;
                        inInstructions = false;
                        inCredits = false;
                    }
                    else if (mainMenu.CurrentlySelectedItem == _credits)
                    {
                        inCredits = true;
                        inMainMenu = false;
                        inOptions = false;
                        inInstructions = false;
                    }
                    else if (mainMenu.CurrentlySelectedItem == _quit)
                    {
                        this.Exit();
                    }
                    else
                    {
                        mainMenu.Visible = true;
                    }
                }


            }
            else if (inInstructions)
            {
                if (!instructionsMenu.MenuItemVisibility(instructionsMenu.CurrentlySelectedItem))
                    instructionsMenu.SelectNext();
                instructionsMenu.Visible = true;
                playerDead = false;

                //set prev and next buttons
                switch (currentInstructionPanel)
                {
                    case 1:
                        if(!instructionsMenu.MenuItemVisibility(_next))
                            instructionsMenu.MenuItemVisibility(_next, true);
                        if(instructionsMenu.MenuItemVisibility(_prev))
                            instructionsMenu.MenuItemVisibility(_prev, false);
                        break;
                    case 2:
                    case 3:
                        if (!instructionsMenu.MenuItemVisibility(_prev))
                            instructionsMenu.MenuItemVisibility(_prev, true);
                        if (!instructionsMenu.MenuItemVisibility(_next))
                            instructionsMenu.MenuItemVisibility(_next, true);
                        break;
                    case 4:
                        if (!instructionsMenu.MenuItemVisibility(_prev))
                            instructionsMenu.MenuItemVisibility(_prev, true);
                        if (instructionsMenu.MenuItemVisibility(_next))
                            instructionsMenu.MenuItemVisibility(_next, false);
                        break;
                    default:
                        if (!instructionsMenu.MenuItemVisibility(_prev))
                            instructionsMenu.MenuItemVisibility(_prev, true);
                        if (!instructionsMenu.MenuItemVisibility(_next))
                            instructionsMenu.MenuItemVisibility(_next, true);
                        break;
                }

                if ((currentKeyboardState.IsKeyDown(Keys.Down) && !previousKeyboardState.IsKeyDown(Keys.Down)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Right) && !previousKeyboardState.IsKeyDown(Keys.Right)))
                {
                    instructionsMenu.SelectNext();
                }
                else if ((currentKeyboardState.IsKeyDown(Keys.Up) && !previousKeyboardState.IsKeyDown(Keys.Up)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Left) && !previousKeyboardState.IsKeyDown(Keys.Left)))
                {
                    instructionsMenu.SelectPrev();
                }

                if (currentKeyboardState.IsKeyDown(Keys.Enter) && !previousKeyboardState.IsKeyDown(Keys.Enter))
                {
                    instructionsMenu.Visible = false;
                    if (instructionsMenu.CurrentlySelectedItem == _prev)
                    {
                        currentInstructionPanel--;
                        if (currentInstructionPanel < 1)
                            currentInstructionPanel = instructionPanelCount;
                    }
                    else if (instructionsMenu.CurrentlySelectedItem == _next)
                    {
                        currentInstructionPanel++;
                        if (currentInstructionPanel > instructionPanelCount)
                            currentInstructionPanel = 1;
                    }
                    else if (instructionsMenu.CurrentlySelectedItem == _returnToMain)
                    {
                        instructionsMenu.SetSelected(_prev);
                        inMainMenu = true;
                        inInstructions = false;
                        inCredits = false;
                        inOptions = false;
                    }
                    else
                        instructionsMenu.Visible = true;
                }
            }

            else if (inOptions)
            {
                optionsMenu.Visible = true;
                playerDead = false;

                if (inColorOptions)
                {
                    optionsMenu.Visible = false;
                    colorOptionsMenu.Visible = true;
                    if ((currentKeyboardState.IsKeyDown(Keys.Down) && !previousKeyboardState.IsKeyDown(Keys.Down)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Right) && !previousKeyboardState.IsKeyDown(Keys.Right)))
                    {
                        colorOptionsMenu.SelectNext();
                    }
                    else if ((currentKeyboardState.IsKeyDown(Keys.Up) && !previousKeyboardState.IsKeyDown(Keys.Up)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Left) && !previousKeyboardState.IsKeyDown(Keys.Left)))
                    {
                        colorOptionsMenu.SelectPrev();
                    }

                    if (currentKeyboardState.IsKeyDown(Keys.Enter) && !previousKeyboardState.IsKeyDown(Keys.Enter))
                    {
                        colorOptionsMenu.Visible = false;

                        if (colorOptionsMenu.CurrentlySelectedItem == _brown)
                        {
                            spriteColor = "";
                        }
                        else if (colorOptionsMenu.CurrentlySelectedItem == _red)
                        {
                            spriteColor = "red";
                        }
                        else if (colorOptionsMenu.CurrentlySelectedItem == _orange)
                        {
                            spriteColor = "orange";
                        }
                        else if (colorOptionsMenu.CurrentlySelectedItem == _yellow)
                        {
                            spriteColor = "yellow";
                        }
                        else if (colorOptionsMenu.CurrentlySelectedItem == _green)
                        {
                            spriteColor = "green";
                        }
                        else if (colorOptionsMenu.CurrentlySelectedItem == _blue)
                        {
                            spriteColor = "blue";
                        }
                        else if (colorOptionsMenu.CurrentlySelectedItem == _purple)
                        {
                            spriteColor = "purple";
                        }
                        else if (colorOptionsMenu.CurrentlySelectedItem == _reset)
                        {
                            spriteColor = "";
                        }
                        else if (colorOptionsMenu.CurrentlySelectedItem == _save)
                        {
                            inColorOptions = false;
                            optionsMenu.Visible = true;
                        }
                        else
                            colorOptionsMenu.Visible = true;
                    }
                }
                else if (inSoundOptions)
                {
                    optionsMenu.Visible = false;
                    soundOptionsMenu.Visible = true;
                    if ((currentKeyboardState.IsKeyDown(Keys.Down) && !previousKeyboardState.IsKeyDown(Keys.Down)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Right) && !previousKeyboardState.IsKeyDown(Keys.Right)))
                    {
                        soundOptionsMenu.SelectNext();
                    }
                    else if ((currentKeyboardState.IsKeyDown(Keys.Up) && !previousKeyboardState.IsKeyDown(Keys.Up)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Left) && !previousKeyboardState.IsKeyDown(Keys.Left)))
                    {
                        soundOptionsMenu.SelectPrev();
                    }

                    if (currentKeyboardState.IsKeyDown(Keys.Enter) && !previousKeyboardState.IsKeyDown(Keys.Enter))
                    {
                        soundOptionsMenu.Visible = false;

                        if (soundOptionsMenu.CurrentlySelectedItem == _on)
                        {
                            playSoundEffects = true;
                            inSoundOptions = false;
                        }
                        else if (soundOptionsMenu.CurrentlySelectedItem == _off)
                        {
                            playSoundEffects = false;
                            inSoundOptions = false;
                            optionsMenu.Visible = true;
                        }
                        else
                        {
                            soundOptionsMenu.Visible = true;
                        }
                    }
                }
                else if (inMusicOptions)
                {
                    optionsMenu.Visible = false;
                    musicOptionsMenu.Visible = true;
                    if ((currentKeyboardState.IsKeyDown(Keys.Down) && !previousKeyboardState.IsKeyDown(Keys.Down)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Right) && !previousKeyboardState.IsKeyDown(Keys.Right)))
                    {
                        musicOptionsMenu.SelectNext();
                    }
                    else if ((currentKeyboardState.IsKeyDown(Keys.Up) && !previousKeyboardState.IsKeyDown(Keys.Up)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Left) && !previousKeyboardState.IsKeyDown(Keys.Left)))
                    {
                        musicOptionsMenu.SelectPrev();
                    }

                    if (currentKeyboardState.IsKeyDown(Keys.Enter) && !previousKeyboardState.IsKeyDown(Keys.Enter))
                    {
                        musicOptionsMenu.Visible = false;

                        if (musicOptionsMenu.CurrentlySelectedItem == _on)
                        {
                            MediaPlayer.Volume = 100;
                            inMusicOptions = false;
                            optionsMenu.Visible = true;
                        }
                        else if (musicOptionsMenu.CurrentlySelectedItem == _off)
                        {
                            MediaPlayer.Volume = 0;
                            inMusicOptions = false;
                            optionsMenu.Visible = true;
                        }
                        else
                        {
                            musicOptionsMenu.Visible = true;
                        }
                    }
                }
                else
                {
                    if ((currentKeyboardState.IsKeyDown(Keys.Down) && !previousKeyboardState.IsKeyDown(Keys.Down)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Right) && !previousKeyboardState.IsKeyDown(Keys.Right)))
                    {
                        optionsMenu.SelectNext();
                    }
                    else if ((currentKeyboardState.IsKeyDown(Keys.Up) && !previousKeyboardState.IsKeyDown(Keys.Up)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Left) && !previousKeyboardState.IsKeyDown(Keys.Left)))
                    {
                        optionsMenu.SelectPrev();
                    }

                    if (currentKeyboardState.IsKeyDown(Keys.Enter) && !previousKeyboardState.IsKeyDown(Keys.Enter))
                    {
                        optionsMenu.Visible = false;
                        if (optionsMenu.CurrentlySelectedItem == _selectSpriteColor)
                        {
                            inColorOptions = true;
                        }
                        else if (optionsMenu.CurrentlySelectedItem == _soundOptions)
                        {
                            inSoundOptions = true;
                        }
                        else if (optionsMenu.CurrentlySelectedItem == _musicOptions)
                        {
                            inMusicOptions = true;
                        }
                        else if (optionsMenu.CurrentlySelectedItem == _returnToMain)
                        {
                            optionsMenu.Visible = false;
                            optionsMenu.SelectNext();
                            inMainMenu = true;
                            inInstructions = false;
                            inCredits = false;
                            inOptions = false;
                        }
                        else
                            optionsMenu.Visible = true;
                    }
                }
            }

            else if (inCredits)
            {
                creditsMenu.Visible = true;
                playerDead = false;

                if (!creditsMenu.MenuItemVisibility(creditsMenu.CurrentlySelectedItem))
                    creditsMenu.SelectNext();

                //set prev and next buttons
                switch (currentCreditsPanel)
                {
                    case 1:
                        if (!creditsMenu.MenuItemVisibility(_next))
                            creditsMenu.MenuItemVisibility(_next, true);
                        if (creditsMenu.MenuItemVisibility(_prev))
                            creditsMenu.MenuItemVisibility(_prev, false);
                        break;
                    case 2:
                        if (!creditsMenu.MenuItemVisibility(_prev))
                            creditsMenu.MenuItemVisibility(_prev, true);
                        if (creditsMenu.MenuItemVisibility(_next))
                            creditsMenu.MenuItemVisibility(_next, false);
                        break;
                    default:
                        if (!creditsMenu.MenuItemVisibility(_prev))
                            creditsMenu.MenuItemVisibility(_prev, true);
                        if (!creditsMenu.MenuItemVisibility(_next))
                            creditsMenu.MenuItemVisibility(_next, true);
                        break;
                }

                if ((currentKeyboardState.IsKeyDown(Keys.Down) && !previousKeyboardState.IsKeyDown(Keys.Down)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Right) && !previousKeyboardState.IsKeyDown(Keys.Right)))
                {
                    creditsMenu.SelectNext();
                }
                else if ((currentKeyboardState.IsKeyDown(Keys.Up) && !previousKeyboardState.IsKeyDown(Keys.Up)) ||
                    (currentKeyboardState.IsKeyDown(Keys.Left) && !previousKeyboardState.IsKeyDown(Keys.Left)))
                {
                    creditsMenu.SelectPrev();
                }

                if (currentKeyboardState.IsKeyDown(Keys.Enter) && !previousKeyboardState.IsKeyDown(Keys.Enter))
                {
                    creditsMenu.Visible = false;
                    if (creditsMenu.CurrentlySelectedItem == _prev)
                    {
                        currentCreditsPanel--;
                        if (currentCreditsPanel < 1)
                            currentCreditsPanel = creditsPanelCount;
                    }
                    else if (creditsMenu.CurrentlySelectedItem == _next)
                    {
                        currentCreditsPanel++;
                        if (currentCreditsPanel > creditsPanelCount)
                            currentCreditsPanel = 1;
                    }
                    else if (creditsMenu.CurrentlySelectedItem == _returnToMain)
                    {
                        creditsMenu.SetSelected(_prev);
                        inMainMenu = true;
                        inInstructions = false;
                        inCredits = false;
                        inOptions = false;
                    }
                    else
                        instructionsMenu.Visible = true;
                }
            }
            else if ((gameOverWin || gameOverLose) && currentKeyboardState.IsKeyDown(Keys.Enter) 
                        && !previousKeyboardState.IsKeyDown(Keys.Enter))
            {
                inMainMenu = true;
                inCredits = false;
                inOptions = false;
                pause = false;
                MediaPlayer.Play(bgMusic[0]);

                gameOverLose = false;
                gameOverWin = false;
                won = false;

                level.Dispose();
                currentLevel = 1;
                /*level = new Level(this, currentLevel);
                level.setCheat(cheatOn);
                level.LoadContent(this, out enemies1, out enemies2, ref player1);*/
            }

            else if (won && currentKeyboardState.IsKeyDown(Keys.Enter) && !previousKeyboardState.IsKeyDown(Keys.Enter))
            {
                won = false;
                level.Dispose();
                levelActive = false;

                totalScore += ((int)timer * 10);
                totalScore += levelScore;
                levelScore = 0;

                if (currentLevel >= MAX_LEVEL)
                {
                    gameOverWin = true;
                }
                else
                {
                    level = new Level(this, ++currentLevel);
                    level.setCheat(clingOn);
                    level.setEnemies(enemiesOn);
                    MediaPlayer.Play(bgMusic[currentLevel]);
                    level.LoadContent(this, out enemies1, out enemies2, ref player1, "squirrel" + ((spriteColor != "") ? "_" + spriteColor : ""));
                    levelActive = true;

                    timer = timers[currentLevel - 1];
                }

            }
            //else if (currentKeyboardState.IsKeyDown(Keys.P) && !previousKeyboardState.IsKeyDown(Keys.P))
            //{
            //    if (pause == false)
            //        pause = true;
            //    else
            //        pause = false;
            //}

            else if (!player1.Dead && !pause && !won )
            {
                if (currentKeyboardState.IsKeyDown(Keys.Escape) && !previousKeyboardState.IsKeyDown(Keys.Escape))
                {
                    MediaPlayer.Play(bgMusic[0]);
                    inMainMenu = true;
                    levelActive = false;

                }
 
                
                if (currentKeyboardState.IsKeyDown(Keys.Right))
                {
                    if (player1.Level_position.X - level.CameraPosition.X >= Game1.CLIENT_WIDTH / 2)
                    {
                        level.MoveCamera(SHIFT);
                    }
                    player1.Move_Right(SHIFT);
                }

                if (currentKeyboardState.IsKeyDown(Keys.Left))
                {
                    if (player1.Level_position.X - level.CameraPosition.X <= Game1.CLIENT_WIDTH / 4)
                    {

                        level.MoveCamera(-SHIFT);
                    }
                    player1.Move_Left(SHIFT, level.CameraPosition.X);
                }

                if (currentKeyboardState.IsKeyDown(Keys.Space) && !previousKeyboardState.IsKeyDown(Keys.Space))
                {
                    if(player1.Jump() && playSoundEffects)
                        soundEffects[(int)soundEffectNames["jump"]].Play();
                }            
            }
            else if (player1.Dead && !pause) // player died
            {
                if (!playerDead)
                {
                    if (playSoundEffects)
                        soundEffects[(int)soundEffectNames["die"]].Play();
                    playerDead = true;
                }

                if (lives == 0 && !won)
                {
                    gameOverLose = true;
                }
                else if (currentKeyboardState.IsKeyDown(Keys.Enter) && !previousKeyboardState.IsKeyDown(Keys.Enter))
                {
                    level.Dispose();
                    if(playSoundEffects)
                        soundEffects[(int)soundEffectNames["redo"]].Play();
                    level = new Level(this, currentLevel);
                    level.setCheat(clingOn);
                    level.setEnemies(enemiesOn);
                    level.LoadContent(this, out enemies1, out enemies2, ref player1, "squirrel" + ((spriteColor != "") ? "_" + spriteColor : ""));
                    playerDead = false;

                    levelScore = 0;
                    timer = timers[currentLevel - 1];
                    lives--;
                }
                else if (currentKeyboardState.IsKeyDown(Keys.Escape) && !previousKeyboardState.IsKeyDown(Keys.Escape))
                {
                    level.Dispose();
                    levelActive = false;
                    //level = new Level(this, currentLevel);
                    //level.LoadContent(this, out enemies1, out enemies2, ref player1);

                    inMainMenu = true;
                    MediaPlayer.Play(bgMusic[0]);
                }
            }


            previousKeyboardState = currentKeyboardState;
            #endregion

            #region:updates
            //Check for enemy collisions with obstacles
            if (levelActive)
            {
                if (!pause)
                {
                    foreach (Enemy1 e1 in enemies1)
                    {
                        if (e1 == null)
                            break;
                        e1.Update(gameTime, graphics);

                        if (!e1.Dead() && player1.Enemy_Collision(e1.BoundingRectangle))
                        {
                            e1.TakeHit();
                            if(playSoundEffects)
                                soundEffects[(int)soundEffectNames["kill"]].Play();
                            levelScore += 100;
                        }
                    }
                    foreach (Enemy2 e2 in enemies2)
                    {
                        if (e2 == null)
                            break;
                        e2.Update(gameTime);

                        if (!e2.Dead && player1.Enemy_Collision(e2.BoundingRectangle))
                        {
                            e2.TakeHit();
                            if(playSoundEffects)
                                soundEffects[(int)soundEffectNames["kill"]].Play();
                            levelScore += 200;
                        }
                    }

                    player1.Update(level.CameraPosition);

                    if (player1.Level_position.Y < level.CameraPosition.Y + (CLIENT_HEIGHT / 4))
                    {
                        level.MoveCameraVertically(-10);
                    }
                    else if (player1.Level_position.Y > level.CameraPosition.Y + ((CLIENT_HEIGHT / 4) * 3))
                    {
                        level.MoveCameraVertically(5);
                    }
                }
                //music.Update();

                base.Update(gameTime);
            #endregion

                if (player1.Finish(level.FinishLine))
                {
                    // WIN - > next level
                    //player1.Jump();
                    won = true;
                }

                if (!won && !player1.Dead && timer > 0 && !pause)
                {
                    // update the timer
                    timer -= (float)(gameTime.ElapsedGameTime.TotalMilliseconds / 1000);
                }
            }
        }
        
        

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            if (inMainMenu)
            {
                spriteBatch.Begin();
                mainMenu.Draw(spriteBatch);
                spriteBatch.End();
            }
            else if (inCredits)
            {
                spriteBatch.Begin();
                creditsMenu.Draw(spriteBatch);
                spriteBatch.DrawString(Fonts[2], _credits, new Vector2(creditsMenu.MenuPosition.X - 10, creditsMenu.MenuPosition.Y - 30), Color.Black);
                switch (currentCreditsPanel)
                {
                    case 1:
                        spriteBatch.Draw(creditsPanels[0], new Vector2(60, 276), Color.White);
                        break;
                    case 2:
                        spriteBatch.Draw(creditsPanels[1], new Vector2(60, 276), Color.White);
                        break;
                    default:
                        spriteBatch.Draw(creditsPanels[0], new Vector2(60, 276), Color.White);
                        break;
                }

                spriteBatch.End();
            }
            else if (inInstructions)
            {
                spriteBatch.Begin();
                instructionsMenu.Draw(spriteBatch);
                spriteBatch.DrawString(Fonts[2], _instructions, new Vector2(instructionsMenu.MenuPosition.X - 10, instructionsMenu.MenuPosition.Y - 30), Color.Black);
                switch (currentInstructionPanel)
                {
                    case 1:
                        spriteBatch.Draw(instructionPanels[0], new Vector2(100, 276), Color.White);
                        break;
                    case 2:
                        spriteBatch.Draw(instructionPanels[1], new Vector2(100, 276), Color.White);
                        break;
                    case 3:
                        spriteBatch.Draw(instructionPanels[2], new Vector2(100, 276), Color.White);
                        break;
                    case 4:
                        spriteBatch.Draw(instructionPanels[3], new Vector2(100, 276), Color.White);
                        break;
                    default:
                        spriteBatch.Draw(instructionPanels[0], new Vector2(100, 276), Color.White);
                        break;
                }

                spriteBatch.End();
            }
            else if (inOptions)
            {
                spriteBatch.Begin();
                optionsMenu.Draw(spriteBatch);
                if (inColorOptions)
                {
                    colorOptionsMenu.Draw(spriteBatch);
                    int choice = 0;
                    switch(spriteColor)       
                    {
                        case "brown":
                            choice = 0;
                            break;
                        case "red":
                            choice = 1;
                            break;
                        case "orange":
                            choice = 2;
                            break;
                        case "yellow":
                            choice = 3;
                            break;
                        case "green":
                            choice = 4;
                            break;
                        case "blue":
                            choice = 5;
                            break;
                        case "purple":
                            choice = 6;
                            break;
                        default:
                            choice = 0;
                            break;
                    }
                    spriteBatch.Draw(spriteChoices[choice], new Rectangle((int)(mainMenu.MenuPosition.X + 240.0f), (int)(mainMenu.MenuPosition.Y + 100.0f), Tile.SIZE, Tile.SIZE), new Rectangle(0,0,Tile.SIZE, Tile.SIZE), Color.White);
                    spriteBatch.DrawString(Fonts[1], "Currently", new Vector2(mainMenu.MenuPosition.X + 290, mainMenu.MenuPosition.Y + 96), Color.Black);
                    spriteBatch.DrawString(Fonts[1], ((spriteColor == "") ? "brown" : spriteColor).Substring(0, 1).ToUpper() + ((spriteColor == "") ? "brown" : spriteColor).Substring(1), new Vector2(mainMenu.MenuPosition.X + 290, mainMenu.MenuPosition.Y + 120), Color.Black);
                }
                else if (inSoundOptions)
                {
                    spriteBatch.Draw(optionsMenu.BGImage, new Rectangle((int)soundOptionsMenu.MenuPosition.X, (int)soundOptionsMenu.MenuPosition.Y - 1, 100, 100), new Rectangle((int)soundOptionsMenu.MenuPosition.X, (int)soundOptionsMenu.MenuPosition.Y, 100, 100), Color.White);
                    soundOptionsMenu.Draw(spriteBatch);
                }
                else if (inMusicOptions)
                {
                    spriteBatch.Draw(optionsMenu.BGImage, new Rectangle((int)musicOptionsMenu.MenuPosition.X, (int)musicOptionsMenu.MenuPosition.Y - 1, 100, 100), new Rectangle((int)musicOptionsMenu.MenuPosition.X, (int)musicOptionsMenu.MenuPosition.Y, 100, 100), Color.White);
                    musicOptionsMenu.Draw(spriteBatch);
                }

                spriteBatch.End();
            }
           
            else
            {
                GraphicsDevice.Clear(Color.CornflowerBlue);
                spriteBatch.Begin();

                if (gameOverWin)
                {
                    spriteBatch.DrawString(Fonts[2], "You win the game!", new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[2].MeasureString("You win the game!").X / 2), 230), Color.White);
                    spriteBatch.DrawString(Fonts[1], "Total Score: " + totalScore, new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[1].MeasureString("Total Score: " + totalScore).X / 2), 280), Color.White);
                    spriteBatch.DrawString(Fonts[1], "Press Enter to return to the menu.", new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[1].MeasureString("Press Enter to return to the menu.").X / 2), 330), Color.White);
                }
                else if (gameOverLose)
                {
                    spriteBatch.DrawString(Fonts[2], "Game Over", new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[2].MeasureString("Game Over").X / 2), 260), Color.White);
                    spriteBatch.DrawString(Fonts[1], "Press Enter to return to the menu.", new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[1].MeasureString("Press Enter to return to the menu.").X / 2), 300), Color.White);
                }
                else
                {
                    level.Draw(spriteBatch);
                    player1.Draw(spriteBatch, level.CameraPosition);

                    foreach (Enemy1 e1 in enemies1)
                    {
                        if (e1 == null)
                            break;
                        if (!e1.Dead())
                            e1.Draw(spriteBatch, level.CameraPosition);
                    }

                    foreach (Enemy2 e2 in enemies2)
                    {
                        if (e2 == null)
                            break;
                        if (!e2.Dead)
                            e2.Draw(spriteBatch, level.CameraPosition);
                    }


                    // draw the lives and timer
                    spriteBatch.DrawString(Fonts[1], "Lives: " + lives, new Vector2(10, 10), Color.White );

                    spriteBatch.DrawString(Fonts[1], "Score: " + (totalScore + levelScore), new Vector2(10, 40), Color.White );

                    spriteBatch.DrawString(Fonts[1], "Time: " + (int)timer, new Vector2(680, 10), Color.White );



                    if (pause)
                    {
                        spriteBatch.DrawString(Fonts[2], "Paused", new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[2].MeasureString("Paused").X / 2), 280), Color.White);
                    }

                    if (player1.Dead)
                    {
                        spriteBatch.DrawString(Fonts[2], "You have died.", new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[2].MeasureString("You have died.").X / 2), 280), Color.White);
                        spriteBatch.DrawString(Fonts[1], "Press Enter to try again.", new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[1].MeasureString("Press Enter to try again.").X / 2), 350), Color.White);
                        spriteBatch.DrawString(Fonts[1], "Press ESC to return to the menu.", new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[1].MeasureString("Press ESC to return to the menu.").X / 2), 400), Color.White);
                    }

                    if (won)
                    {
                        spriteBatch.DrawString(Fonts[2], "You have won!", new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[2].MeasureString("You have won!").X / 2), 200), Color.White);
                        spriteBatch.DrawString(Fonts[1], "Time Bonus = " + ((int)timer * 10), new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[1].MeasureString("Time Bonus = " + ((int)timer * 10)).X / 2), 250), Color.White);
                        spriteBatch.DrawString(Fonts[1], "Press Enter to go on.", new Vector2(CLIENT_WIDTH / 2 - (int)(Fonts[1].MeasureString("Press Enter to go on.").X / 2), 350), Color.White);
                    }
                }

                spriteBatch.End();
            }
        }
    }
}