using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace SuperScooter
{
    public class player : Collideable
    {
        #region Constants
        public const int Y_DISPLACEMENT_FOR_TILE_GRAPHICS = 5;
        #endregion

        #region:variables
        private Texture2D player_image;
        //Added by Rae Melton----------
        private Texture2D player_sprite_dummy;
        private Sprite player_sprite;
        //-----------------------------
        private Game game;
        //private bool player;
        private bool dead;
        private bool jumping;
        private bool player_on_ground;
        private int jump_speed = 10;
        private int gravity = 5;
        private int max_jump = 14;
        private int amount_player_has_jumped;
        private int old_hieght;
        private bool cling;

        private List<List<Tile>> tiles;
        Tile left = new Tile(null, CollideType.Passable, 0, 0);
        Tile right = new Tile(null, CollideType.Passable, 0, 0);
        Tile top = new Tile(null, CollideType.Passable, 0, 0);
        Tile top_right = new Tile(null, CollideType.Passable, 0, 0);
        Tile top_left = new Tile(null, CollideType.Passable, 0, 0);
        Tile bottom = new Tile(null, CollideType.Passable, 0, 0);
        Tile bottom_right = new Tile(null, CollideType.Passable, 0, 0);
        Tile bottom_left = new Tile(null, CollideType.Passable, 0, 0);
        #endregion

        #region:constructors

        /// <summary>
        /// creates a player with knowledge of terrain on the level and were the player is to begein on the level
        /// </summary>
        public player(Game game, Vector2 tile_position, string player_image_name, ref List<List<Tile>> tiles)
        {
            this.game = game;
            this.tiles = tiles;
            LoadContent(player_image_name);
            Initialize();
            SetPosition(tile_position);

            //Added by Rae Melton----------
            //avatar = player_image;
            avatar = player_sprite_dummy;
            //-----------------------------
        }
        #endregion

        #region:initialize()
        protected void Initialize()
        {
            dead = false;
            jumping = false;
            player_on_ground = true;
            amount_player_has_jumped = 0;
            cling = false;
        }
        #endregion

        #region:loadcontent
        /// <summary>
        /// loads the player images
        /// </summary>
        private void LoadContent(string player_image_name)
        {
            player_image = game.Content.Load<Texture2D>("Textures/" + player_image_name);
            player_sprite_dummy = game.Content.Load<Texture2D>("Textures/squirrel_still");

            //Added by Rae Melton----------
            player_sprite = new Sprite(game, player_image, Sprite.Direction.RIGHT, Sprite.Direction.RIGHT, Tile.SIZE, Tile.SIZE, 10, 4, 3, 0);
            //-----------------------------
        }
        #endregion

        #region:GET/SET

        /// <summary>
        /// returns if the player is dead or not
        /// </summary>
        public bool Dead
        {
            get { return dead; }
        }

        /// <summary>
        /// gives the players current location on the screen
        /// </summary>
        public Point Level_position
        {
            get { return shell.Location; }
        }
        #endregion

        #region:movement

        #region:jump
        /// <summary>
        /// the player will jump on the screen. If the player is already jumping or is not on solid ground player will not jump
        /// </summary>
        public bool Jump()
        {
            if (player_on_ground)
            {
                if (cling)
                {
                    player_sprite.FlipSpriteRightSideUp();
                    cling = false;
                }
                else
                {
                    jumping = true;
                }
                player_on_ground = false;
                return true;
            }
            return false;
        }
        #endregion

        #region:Move left
        /// <summary>
        /// player checks to see if he is colliding with any objects on its left.
        /// if not the player on the screen and his position on the level move left
        /// </summary>
        /// <param name="shift">Number of pixils the player will</param>
        public void Move_Left(int shift, int CameraPosition)
        {
            //when moving the graphic for left and right must be switched if the player image is upside down
            if (cling)
            {
                //Added by Rae Melton-Appended by Daniel Tate
                player_sprite.CurrentDirection = Sprite.Direction.RIGHT;
                player_sprite.ChangeState();
                //-----------------------------
            }
            else
            {
                //Added by Rae Melton----------
                player_sprite.CurrentDirection = Sprite.Direction.LEFT;
                player_sprite.ChangeState();
                //-----------------------------
            }

            if (shell.X > CameraPosition)
            {
                if (cling)
                {
                    if (top.collideType != CollideType.Impassable)
                    {
                        cling = false;
                        player_sprite.FlipSpriteRightSideUp();
                    }
                }
                if (!Collision(left) && (shell.Y == left.UpperBound || !Collision(bottom_left)))
                {
                    ChangePosition(new Vector2(-shift, 0));
                }
                while (Collision(left) || Collision(bottom_left))
                {
                    ChangePosition(new Vector2(1, 0));
                }
            }
        }
        #endregion

        #region:Move right
        /// <summary>
        /// player checks to see if he is colliding with any objects on its right.
        /// if not the player on the screen and his position on the level move right
        /// </summary>
        public void Move_Right(int shift)
        {
            //when moving the graphic for left and right must be switched if the player image is upside down
            if (cling)
            {
                //Added by Rae Melton-Appended by Daniel Tate
                player_sprite.CurrentDirection = Sprite.Direction.LEFT;
                player_sprite.ChangeState();
                //-----------------------------
            }
            else
            {
                //Added by Rae Melton----------
                player_sprite.CurrentDirection = Sprite.Direction.RIGHT;
                player_sprite.ChangeState();
                //-----------------------------
            }

            if (cling)
            {
                if (top_right.collideType != CollideType.Impassable)
                {
                    cling = false;
                    player_sprite.FlipSpriteRightSideUp();
                }
            }
            if (!Collision(right) && (shell.Y == right.UpperBound || !Collision(bottom_right)))
            {
                ChangePosition(new Vector2(shift, 0));
            }
            while (Collision(right) || Collision(bottom_right))
            {
                ChangePosition(new Vector2(-1, 0));
            }
        }
        #endregion

        #region:Move up
        /// <summary>
        /// player checks to see if he is colliding with any objects above him.
        /// The players tile will move up the level and screen.
        /// </summary>
        /// <param name="shift">number of pixils player will move up on the screen</param>
        /// <returns>True if player can move. False he cant.</returns>
        private bool Move_Up(int shift)
        {
            if (!Collision(top) && !Collision(top_right))
            {
                ChangePosition(new Vector2(0, -shift));

                while (Collision(top) || Collision(top_right))
                {
                    ChangePosition(new Vector2(0, 1));
                    cling = true;
                }
                if (cling)
                {
                    if (top.collideType != CollideType.Impassable || top_right.collideType != CollideType.Impassable)
                    {
                        cling = false;
                    }
                    else
                        player_sprite.FlipSpriteUpSideDown();
                    return false;
                }
                else
                    return true;
            }
            return false;
        }
        #endregion

        #region:Move down
        /// <summary>
        /// player checks to see if he is colliding with any objects below him.
        /// The players tile will move down the level and screen screen.
        /// </summary>
        /// <param name="shift">number of pixils player will move down on the screen</param>
        /// <returns>True if player can move. False he cant.</returns>
        private bool Move_Down(int shift)
        {
            if (!cling)
            {
                //this if statment will move the player down if he is not colliding with a block or standing ontop of a block
                if (((bottom.collideType == CollideType.Passable || bottom.collideType == CollideType.Boundary || bottom.collideType == CollideType.Enemy2Platform || shell.Bottom != bottom.UpperBound) && (!Bottom_Collision(bottom_right))) && ((bottom_right.collideType == CollideType.Passable || bottom_right.collideType == CollideType.Boundary || bottom_right.collideType == CollideType.Enemy2Platform || shell.Bottom != bottom_right.UpperBound - 1) && !Bottom_Collision(bottom)))
                {
                    ChangePosition(new Vector2(0, shift));
                    player_on_ground = false;

                    //if player has collided with a object after move back the player back out
                    while (Bottom_Collision(bottom) || Bottom_Collision(bottom_right))
                    {
                        ChangePosition(new Vector2(0, -1));
                        player_on_ground = true;
                    }
                    return true;
                }
            }
            player_on_ground = true;
            return false;
        }
        #endregion

        #endregion

        #region:update
        /// <summary>
        /// updates player on the screen
        /// used for jumping and animating player
        /// </summary>
        public void Update(Point cameraPosition)
        {
            #region:update surrounding tiles

            if (shell.Top >=  cameraPosition.Y + Game1.CLIENT_HEIGHT)
            {
                player_sprite.KillSprite();
                dead = true;
            }

            //finds the player tile
            int tilePositionX = (int)shell.X / 40;
            int tilePositionY = (int)shell.Y / 40;

            //keeps the player from crashing if he tries to get a tile outside of the level
            try
            {
                //updates all tiles around the players tile
                right = tiles[tilePositionX + 1][tilePositionY];
                left = tiles[tilePositionX - 1][tilePositionY];
                top = tiles[tilePositionX][tilePositionY - 1];
                top_right = tiles[tilePositionX + 1][tilePositionY - 1];
                bottom = tiles[tilePositionX][tilePositionY + 1];
                bottom_right = tiles[tilePositionX + 1][tilePositionY + 1];
                bottom_left = tiles[tilePositionX - 1][tilePositionY + 1];
            }
            catch (Exception e)
            {
                string trash = e.Message;
            }
            #endregion

            #region:jump
            //disables gravity until max jump hight or player hits a object
            if (jumping && !dead)
            {
                if (amount_player_has_jumped < max_jump && Move_Up(jump_speed))
                {
                    amount_player_has_jumped++;
                }
                else
                {
                    amount_player_has_jumped = 0;
                    jumping = false;
                }
            }
            else
            {
                Move_Down(gravity);
            }
            #endregion

            #region Rae's Animation changes >.>
            //Added by Rae Melton----------
            //if the player is not moving left or right, return the standing sprite state.
            if(!dead)
            if (Keyboard.GetState().IsKeyUp(Keys.Left) && Keyboard.GetState().IsKeyUp(Keys.Right))
                player_sprite.StandSprite();
            //-----------------------------
            #endregion
        }
        #endregion

        #region:collision
        /// <summary>
        /// returns if the player has collided with a object (this function considers passable objects and platforms to never collide with player)
        /// </summary>
        private bool Collision(Tile tile)
        {
            if (tile.collideType == CollideType.Passable || tile.collideType == CollideType.Platform || tile.collideType == CollideType.Boundary || tile.collideType == CollideType.Enemy2Platform)
                return false;
            else
                return shell.Intersects(tile.BoundingRectangle);
        }

        /// <summary>
        /// return if the player has collidd with a object with function will only return that you have collided with a passible object if you are at the very top of it)
        /// </summary>
        private bool Bottom_Collision(Tile tile)
        {
            if (tile.collideType == CollideType.Passable || tile.collideType == CollideType.Boundary || tile.collideType == CollideType.Enemy2Platform || (tile.collideType == CollideType.Platform && tile.UpperBound < shell.Bottom - 10))
                return false;
            else
                return shell.Intersects(tile.BoundingRectangle);
        }

        /// <summary>
        /// returns true if the player has entered the center of the finish tile
        /// </summary>
        public bool Finish(Point finish_line)
        {
            if (shell.Bottom > finish_line.Y && shell.Top < finish_line.Y && shell.Left < finish_line.X && shell.Right > finish_line.X)
                return true;
            else
                return false;
        }

        /// <summary>
        /// detects of a player has collided with an enemy
        /// </summary>
        /// <param name="game_object">a rectangle of a object being tested on the screen</param>
        /// <returns>true if there is a collision/false if there is none</returns>
        public bool Enemy_Collision(Rectangle game_object)
        {
            if (shell.Intersects(game_object))
            {
                if (old_hieght < game_object.Top + 21)
                {
                    jumping = false;
                    player_on_ground = true;
                    Jump();
                    return true;
                }
                else
                {
                    player_sprite.KillSprite();
                    dead = true;
                }
            }
            old_hieght = shell.Bottom;
            return false;
        }
        #endregion

        #region:draw
        /// <summary>
        /// draws player to screen
        /// </summary>
        public void Draw(SpriteBatch spriteBatch, Point CameraPosition)
        {

            player_sprite.Draw(spriteBatch, (int)(shell.X - CameraPosition.X), (int)(shell.Y - CameraPosition.Y + Y_DISPLACEMENT_FOR_TILE_GRAPHICS), Color.White);
        }
        #endregion
    }
}